/*
 *  JAPEBackRefs.java
 *
 *  $Id: Transducer.java 13256 2010-12-04 15:35:05Z johann_p $
 */

package at.ofai.gate.japebackrefs;

import gate.creole.ResourceInstantiationException;
import java.util.*;

import gate.*;
import gate.creole.metadata.*;
import gate.jape.constraint.ConstraintPredicate;


/** 
 * A version of the GATE JAPE Transducer that already has the necessary
 * constraint predicate predefined to allow backwards value references using
 * the language construct
 * <pre>{Token valueref {Token.string != "ref1"}} {Token} {Token valueref {Token.string == "ref1"}}</pre>
 *
 * @author Johann Petrak
 */
@CreoleResource(name = "JAPEBackRefs",
        comment = "JAPE Transducer with preconfigured backref capabilities")
public class JAPEBackRefs  extends gate.creole.Transducer
  implements ProcessingResource {

  @Override
  public Resource init() throws ResourceInstantiationException {
    //super.init();
    String opName = "at.ofai.gate.japebackrefs.ValueRef";
    Class<? extends ConstraintPredicate> clazz = null;
    try {
      clazz = Class.forName(opName, true, Gate.getClassLoader()).asSubclass(ConstraintPredicate.class);
    } catch (ClassNotFoundException e) {
      //if couldn't find it that way, try with current thread class loader
      try {
        clazz = Class.forName(opName, true,
          Thread.currentThread().getContextClassLoader()).asSubclass(ConstraintPredicate.class);
      } catch (ClassNotFoundException e1) {
        throw new ResourceInstantiationException("Cannot load class for operator: " + opName, e1);
      }
    } catch (ClassCastException cce) {
      throw new ResourceInstantiationException("Operator class '" + opName + "' must implement ConstraintPredicate");
    }
    //instantiate an instance of the class so can get the operator string
    try {
      ConstraintPredicate predicate = clazz.newInstance();
      String opSymbol = predicate.getOperator();
      //now store it in ConstraintFactory
      Factory.getConstraintFactory().addOperator(opSymbol, clazz);
    } catch (Exception e) {
      throw new ResourceInstantiationException("Cannot instantiate class for operator: " + opName, e);
    }
    super.init();
    return this;
  }

} // class JAPEBackRefs
